import re, glob, datetime, sys
import numpy
import pdb

def get_item(l, i=0):
    return l[i]

def get_cover(l):
    return numpy.mean([l["nbn"][i]/float(l["totn"][i]) for i in range(len(l["totn"]))])

def get_degree(l):
    return numpy.mean([l["tot_degree"][i]/float(l["nbc"][i]) for i in range(len(l["totn"]))])


def make_ibool(v):
    try:
        return int(int(v)==1)
    except ValueError:
        return 0

def make_time(v):
    tmp = re.match("\s*(?P<h>[0-9][0-9]*):(?P<m>[0-9][0-9]):(?P<s>[0-9][0-9]).(?P<ms>[0-9][0-9]*)\s*$", v)
    if tmp is not None:
        return int(tmp.group("h"))*3600+int(tmp.group("m"))*60+int(tmp.group("s"))+float("0."+tmp.group("ms"))
    return 0

def make_str(v):
    if v is not None:
        return v.replace("_"," ")
    

if len(sys.argv) > 1:
    suff = "_"+sys.argv[1]
else:
    suff = "ZZ"

MAP_TYPS = {"f": float, "s": make_str, "d": int, "b": make_ibool, "t": make_time}
SEP = {"d": "$", "f": "$"}
ALIGN = {"d": "r", "f": "r"}

filter_conds = {"DOS": {"lmbdi": ["0.001", "0.01", "0.1", "1.0"] },
                "MAR": {"lmbdi": ["0.25", "0.5", "0.75", "0.95"] }}
pattn = "_l[-+]?(?P<lmbdi>(\d+(\.\d*)?))-(?P<suff>[^\.]*)(-[GfF](?P<from>[0-9]*))?\.(?P<algo>.*)_(?P<k>20)$"

groups = {"all": 
          [("dblp_E2", "\\DBLPed{}",["dblpE2"], ["^wegonet_(?P<dataset>.*)2-0.3"]),
           ("dblp_L", "\\DBLPcf{}",["dblpL"], ["^author-author-wpapers.(?P<dataset>.*)_all"]),
           ("gplus_SM", "\\GPlusSM{}",["gplusS"], ["^(?P<dataset>[0-9]*)", "^(?P<dataset>[0-9]*)"]),
           ("gplus_LX", "\\GPlusXL{}",["gplusL"], ["^(?P<dataset>[0-9]*)", "^(?P<dataset>[0-9]*)"]),
           ("facebook", "\\Fb{}",["facebook"], ["^(?P<dataset>[0-9]*)"]),
           ("BKGW", "\\BKGW{}",["BKGW"], ["^BKdbboxL?_(?P<dataset>.*)", "^GWdbboxL?_(?P<dataset>.*)"])]
}

methods = [("DOS", "\\algDOS{}"), ("MAR", "\\algMAR{}")]
methods_lbl = dict(methods)
collections_lbl = {"gplus_SM": "\\GplusSM", "gplus_XL": "\\GplusXL", "dblp_L": "\\DBLPed", "dblp_E2": "\\DBLPed" }
# 

series_fields = ["algo", "k"]
group_fields = ["algo", "k", "lmbdi"]
sort_fields = ["algo", "k", "slmbd", "totn", "dataset"] #, "lmbd"]

out_fields = [("#", "$\\#$", "algo", len, None, "d"),
              ("lmbdi", "$\\lambda_i$", "lmbdi", get_item, {"i": 0}, "s"),
              ("nbc", "$k_r$", "nbc", numpy.mean, None, ".2f"),
              ("avg_degree", "$\\delta$",None, get_degree, None,".2f"),
              ("tot_degree", "$T_{\\delta}$","tot_degree", numpy.mean, None,".2f"),
              ("tot_dist", "$T_{d}$", "tot_dist", numpy.mean, None, ".2f"),
              ("avg_jacc", "$J_{\\sim}$","avg_jacc", numpy.mean, None, ".2f"),
              ("modC", "$Q_{C}$","modC", numpy.mean, None, ".2f"),
              ("modE", "$Q_{E}$","modE", numpy.mean, None, ".2f"),
              ("min_size", "$|C_i|_{-}$","min_size", numpy.min, None, "d"),
              ("max_size", "$|C_i|_{+}$","max_size", numpy.max, None, "d"),
              ("avg_size", "$|C_i|_{\\sim}$","avg_size", numpy.mean, None, "d"),
              ("nbn", "$C$", None, get_cover, None, ".2f"),
              ("max_mltn", "$m_{+}$","max_mltn", numpy.max, None, ".2f"),
              ("avg_mltn", "$m_{\\sim}$","avg_mltn", numpy.mean, None, ".2f"),
              ("tot_time", "$t(s)$","tot_time", numpy.mean, None,".2f")
              ]

out_fields = [("lmbdi", "$\\bal{}$", "lmbdi", get_item, {"i": 0}, "s"),
              ("nbc", "$\\abs{\\solset{}}$", "nbc", numpy.mean, None, "d"),
              ("avg_degree", "$\\delta$",None, get_degree, None,".2f"),
              ("avg_dist", "$\\dist{}$", "avg_dist", numpy.mean, None, ".2f"),
              ("avg_djacc", "$\jacc{}$","avg_djacc", numpy.mean, None, ".2f"),
              ("modE", "$\\modE{}$","modE", numpy.mean, None, ".3f"),
              ("nbn", "$\\cover{}$", None, get_cover, None, ".2f"),
              ("avg_size", "$\\abs{\\solw{}}$","avg_size", numpy.mean, None, "d"),
              ("avg_mltn", "$\\mltpl{v}$","avg_mltn", numpy.mean, None, ".2f")
              ]



root_rep = "ROOT_REP/"
xps_rep = root_rep + "xps/"

ext_c = ".communities"

line_out = "X SUMMARY\ttot_score=%(tot_score)f tot_degree=%(tot_degree)f tot_dist=%(tot_dist)f lmbd=%(lmbd)f nbc=%(nbc)d nbn=%(nbn)d totn=%(totn)d min_jacc=%(min_jacc)f max_jacc=%(max_jacc)f avg_jacc=%(avg_jacc)f min_size=%(min_size)f max_size=%(max_size)f avg_size=%(avg_size)f max_mltn=%(max_mltn)f avg_mltn=%(avg_mltn)f modC=%(modC)f modE=%(modE)f tot_time=%(tot_time)s\n"

PREAMBLE_DOC="""\\documentclass[a4paper,10pt]{article}
\\usepackage[top=1cm, bottom=1cm, left=1cm, right=1cm]{geometry}
\\usepackage{booktabs}
\\begin{document}
"""

PREAMBLE_TBL="""
\\begin{table}[t]
\\centering
\\caption{Results for %s, $k=20$.}
\\label{tab:stats_%s}
\\footnotesize
\\begin{tabular}{%s}
\\toprule
"""

POSTAMBLE_TBL="""
\\bottomrule
\\end{tabular}
\\end{table}
"""

POSTAMBLE_DOC="""
\\end{document}
"""

PREAMBLE_DOC=""
POSTAMBLE_DOC=""

def gather_fields(line_out):
    parts = line_out.strip().split("\t")
    patt = parts[0]
    fields = []
    for piece in parts[1].split(" "):
        tmp = re.match("^(?P<field_out>[^\t =]*)=\%\((?P<field_in>[^\t =]*)\)(?P<format>[a-z])$", piece)
        if tmp is not None:
            fo, fi, ff = (tmp.group("field_out"), tmp.group("field_in"), tmp.group("format"))
            if ff == "s":
                if re.search("time", fi) or re.search("time", fo):
                    ff = "t"
            fields.append((fo,fi,ff))
    return fields, patt

def read_stats(fields_dict, pattl, serie, pattn, filter_conds, rep_base="../xps/", ext=".txt", fil="*"):
    results = []
    pattern = rep_base+serie+"/"+fil+ext
    for filename in glob.glob(pattern):
        if "snap" in serie:
            sndict = parse_shortname(filename[len(rep_base+serie)+1:-len(ext)], pattn.replace("<k>20", "<k>10"))
        elif ("Sahara" in filename) or ("India" in filename):
            sndict = {}
        else:
            sndict = parse_shortname(filename[len(rep_base+serie)+1:-len(ext)], pattn)
        if len(sndict) > 0 and filter_sndict(sndict, filter_conds):
            out = {}
            sndict["serie"] = serie
            with open(filename) as fp:
                for line in fp:
                    if re.match(pattl, line):
                        parts = line.strip().split("\t")[1].split()
                        for part in parts:
                            tt = part.split("=")
                            if tt[0] in fields_dict:
                                out[tt[0]] = fields_dict[tt[0]](tt[1])
            if len(out) > 0:
                sndict.update(out)
                if "lmbdi" in sndict and "algo" in sndict:
                    sndict["slmbd"] = 2*(0.5-(re.match("[DM][OA][SR]", sndict["algo"]) is not None))*float(sndict["lmbdi"])
                sndict["avg_djacc"] = 1-sndict["avg_jacc"]
                sndict["avg_dist"] = 2.*sndict["tot_dist"]/((sndict["nbc"]-1)*sndict["nbc"])
                # sndict["modE"] = 10**2 * sndict.get("modE",-10)
                results.append(sndict)
    return results

def parse_shortname(shortname, pattern):
    tmp = re.search(pattern, shortname)
    if tmp is not None:
        return dict([(t, make_str(v)) for t,v in tmp.groupdict().items()])
    else:
        return {}

# def filter_sndict(sndict, filter_conds={}):
#     for k, vs in filter_conds.items():
#         if sndict.get(k, None) not in vs:
#             return False
#     return True

def filter_sndict(sndict, filter_conds={}):
    for k, vs in filter_conds.get(sndict["algo"],{}).items():
        if sndict.get(k, None) not in vs:
            return False
    return True

def prepareOut(accumulate, acc_fields):
    prep = {}
    for fto, _, ffrom, ffun, args, _ in acc_fields:
        if ffrom is None:
            prep[fto] = ffun(accumulate)
        elif args is not None:
            prep[fto] = ffun(accumulate[ffrom], **args)
        else:
            try:
                prep[fto] = ffun(accumulate[ffrom])
            except KeyError:
                pdb.set_trace()
                print len(accumulate)
    return prep



fields, pattl = gather_fields(line_out)
ff_dict = dict([(f[0], MAP_TYPS[f[-1]]) for f in fields])

algcols = "@{\\hspace*{0.5em}}".join([ALIGN.get(f[-1][-1], "l") for f in out_fields])
stpfrt = " & ".join([SEP.get(f[-1][-1], "")+"%("+f[0]+")"+f[-1]+SEP.get(f[-1][-1],"") for f in out_fields])

for gg, series in groups.items():
    filename_out = xps_rep+ "../compactX_"+gg+".tex" #_"+ datetime.datetime.now().strftime('%Y%m%d%H%M%S') + ".txt"
    fo = open(filename_out, "w")
    # fo = sys.stdout
    fo.write(PREAMBLE_DOC)
    for serie_l, serie_t, seriex, patttx in series:
        results = []
        for i in range(len(seriex)):
            results.extend(read_stats(ff_dict, pattl, seriex[i], patttx[i]+pattn, filter_conds, rep_base=xps_rep, ext=ext_c, fil="*"))
        try:
            results.sort(key=lambda x:[x[sf] for sf in sort_fields])
        except KeyError:
            pdb.set_trace()
            print len(results)
        fo.write(PREAMBLE_TBL % (serie_t, serie_l, algcols))
        fo.write(" & ".join(["\\multicolumn{1}{c}{"+f[1]+"}" for f in out_fields]) + " \\\\\n\midrule\n")
        # fo.write(" & ".join([f[1] for f in out_fields]) + " \\\\\n\midrule\n")
        countl = 0
        accumulate = {}
        for ri, result in enumerate(results):                
            if len(accumulate) > 0 and sum([results[ri-1][kt] != result[kt] for kt in group_fields]) > 0:
                prep = prepareOut(accumulate, out_fields)
                fo.write(stpfrt % prep)
                fo.write(" \\\\\n")
                accumulate = {}
            if ri == 0 or sum([results[ri-1][kt] != result[kt] for kt in series_fields]) > 0:
                if ri > 0:
                    fo.write("[0.6em]\n")
                fo.write("\\multicolumn{%d}{c}{ %s }\\\\[0.2em]\n" % (len(out_fields), methods_lbl.get(result["algo"], result["algo"])))
            for k,v in result.items():
                if k in accumulate:
                    accumulate[k].append(v)
                else:
                    accumulate[k] = [v]
        if len(accumulate) > 0:
            prep = prepareOut(accumulate, out_fields)
            fo.write(stpfrt % prep)
            fo.write(" \\\\\n")
        fo.write(POSTAMBLE_TBL)
    fo.write(POSTAMBLE_DOC)
    fo.close()
